
; Actually the code is public domain, but the collection
; of header signatures is licensed under the GPL2 :-)

; Compile: nasm -o filetype.com filetype.asm
; Contact: ericBARcoli.uni-sb.de (BAR=@)

; Errorlevels:
; 0 okay 1 syntax error 2 failed, no wildcards 3 no files match
; (if wildcards used, read failure does not affect errorlevel)

%define buffer 0x80	; re-use command line option buffer
	; please put optional filetypes in the list in blocks
	; with %if larger...%endif to keep default .com small!
%define larger 0	; 1 if > 2 kB UPXed filetype.com okay

	org 100h
start:	mov dx,finddta	; offset
	mov ah,0x1a
	int 0x21
	mov si,81h
	mov di,buffer	; offset
	cld
sname:	lodsb
	cmp al,13	; CR marks end of command line
	jz ename	
	cmp al,' '
	jz sname	; skip leading space
	jb ename
	cmp al,'/'	; user tried to use an option?
	jz noname
	cmp al,'?'
	jnz nowld1
	inc word [wildcards]
nowld1:	cmp al,'*'
	jnz nowld2
	inc word [wildcards]
nowld2:	cmp al,'\'
	jnz nobck1
	inc word [pathspecs]
nobck1:	stosb		; first char of name

sname2:	lodsb
	cmp al,'?'
	jnz nowld3
	inc word [wildcards]
nowld3:	cmp al,'*'
	jnz nowld4
	inc word [wildcards]
nowld4:	cmp al,':'
	jnz nocol
	inc word [pathspecs]
nocol:	cmp al,'\'
	jnz nobck2
	inc word [pathspecs]
nobck2:	cmp al,13
	jz ename
	cmp al,' '
	jbe ename
	stosb
	jmp short sname2

ename:	mov al,0	; terminate string
	stosb
	cmp [buffer],al	; string empty?
	jz noname
	mov dx,buffer
	cmp word [wildcards], byte 0
	jz loadfile
	jmp dofindfirst	; returns with offset dx to loadfile

noname:	mov dx,usage
	mov ah,9	; show string
	int 0x21
	mov ax,0x4c01	; exit
	int 0x21

loadfile:
	; mov dx,buffer	; offset
	call showname
	mov ax,0x3d00	; open for reading
	int 0x21
	jc ferror
	mov bx,ax	; handle
	mov dx,buffer	; re-use command line option buffer
	mov cx,0x7f	; read at most that many bytes
	mov ah,0x3f	; read from file
	int 0x21
	jc ferr2
	add ax,dx	; calculate offset of 1st unfilled byte
	mov di,ax
	mov al,0
fill:	stosb
	cmp di,start	; pad until at most there
	jbe fill
	mov ah,0x3e	; close file (handle bx)
	int 0x21
	; ignore errors
	call analyze	; ### check which file type... ###
	mov dx,notype	; offset
	jc unktyp	; CY set if no known type
	mov dx,crlf
unktyp:	mov ah,9	; show string
	int 0x21
	cmp word [wildcards], byte 0
	jnz nextfile
	mov ax,0x4c00	; exit
	int 0x21

nextfile:
	mov ah,0x4f	; findnext
	int 0x21
	mov dx,finddta+0x1e	; offset of find result
	jc allfiles
	jmp loadfile
allfiles:
	mov dx,eofiles
	mov ah,9	; show string
	int 0x21
	mov ax,0x4c00	; exit
	int 0x21
	
ferr2:	mov ah,0x3e	; close file (handle bx)
	int 0x21
	; ignore errors
ferror:	mov dx,badfile	; failed to open string
	mov ah,9	; show string
	int 0x21
	cmp word [wildcards], byte 0
	jz ferrx	; no wildcards? -> all failed, show help
	jmp short nextfile
ferrx:	mov dx,usage
	mov ah,9	; show string
	int 0x21
	mov ax,0x4c02	; exit
	int 0x21


dofindfirst:
	cmp word [pathspecs],byte 0
	jnz ferrx	; complain: must not use wildcards w/ path
	mov dx,buffer	; offset
	mov ax,0x4e00	; findfirst
	mov cx,0x27	; include archive, system, hidden, read-only
	int 0x21
	mov dx,finddta+0x1e	; offset of find result
	jc ferr3
	jmp loadfile
ferr3:	mov dx,nofiles
	mov ah,9	; show string
	int 0x21
	mov ax,0x4c03	; exit
	int 0x21

showname:		; show 0 terminated string at DX
	push si
	push dx
	push cx
	xor cx,cx
	mov si,dx
show:	lodsb
	or al,al
	jz shown
	mov ah,2	; write char to stdout
	mov dl,al
	int 0x21
	inc cx
	jmp short show
shown:	cmp cx,12	; less than 8+1+3 chars yet? then pad!
	jae sep
	mov ah,2	; write char
	mov dl,' '
	int 0x21
	inc cx
	jmp short shown
sep:	mov ah,2	; write char
	mov dl,':'
	int 0x21
	mov ah,2	; write char
	mov dl,' '
	int 0x21
	pop cx
	pop dx
	pop si
	ret

wildcards:		; count can be at most ca 0x7f anyway
	dw 0		; number of wildcard chars encountered
pathspecs:
	dw 0		; number of backslashes and : encountered
	
badfile	db "Cannot open file!",13,10,13,10,"$"
nofiles	db "No files found!",13,10,13,10,"$"
eofiles	db "Done for all files.",13,10,13,10,"$"
notype	db "other file"
crlf	db 13,10,"$"

usage	db "Free GPL2 file header / file type checker tool"
	db " by Eric Auer 2007",13,10
	db "Usage: FILETYPE filename         See www.gnu.org"
	db " for GPL license",13,10
	db "Filename can contain wildcards -if- it contains"
	db " no drive or path",13,10,"$"


analyze:	; find out which file type matches the buffer
	mov bx,types	; offset
nexttype:
	mov si,[bx]
	inc bx
	inc bx
	or si,si
	jnz dotype	; all types tried?
	stc
	ret
dotype:	xor ax,ax
	lodsb
	cmp al,-1	; all strings matched?
	jz matched
	add ax,buffer	; find offset to look at
	mov di,ax	; buffer pointer
	xor ax,ax
	lodsb
	mov cx,ax	; find count for comparison
	repz cmpsb	; compare strings
	jnz nexttype
	jcxz dotype	; yes, we matched one string :-)
	jmp short nexttype	; try other types
matched:
	lodsb		; skip size
	mov dx,si	; file type name string offset
	mov ah,9	; show string
	int 0x21
	clc
	ret

types	dw flash1, flash2, html1, html2, html3, mpeg1, mpeg2
	dw bmp, bzip2, aifc, aiff, gif87a, gif89a, mp3id3, tiffi
	dw tiffm, cab, midi, exemz, tex, ogg, pdf12, pdf13, pdf14
	dw pdf15, pdf16, pdf1x, zip, epscr, pscr2, pscr3, rar
	dw avir, midir, wavpcm, realm, rtf, sunmu, sun8, sun16
	dw pcx, gzip, wmv, elf, x86obj, png, jser, jclass, msoff
	dw word2, utfbom, dvi, jjfif, jexif, mp3f2, mp3f3, mp3fa
	dw mp3fb, ffff, xml
%if larger
	dw xpm
%endif
	dw lha, lzh, movfree, mp4v, mp4a
	dw mmdat, mcmov, mmvhd, mwide, kernel, kern0, upx1, upx2
	dw xmmod, st3mod
%if larger
	dw st2mod1, st2mod2
%endif
	dw fli
	dw 0
	; TAR signature would be "ustar" at offset 0x101, bit late
	; PE EXE signature is often at offset 0x80 (see word at 0x3c)

	; Format of each entry:
	; repeated [ offset, length, match-string ]
	; offset -1 is description string shown if all match
	; description string needs no length info
flash1	db 0, 3, "CWS"
	db -1, 0, "Macromedia Flash compressed$"
flash2	db 0, 3, "FWS"
	db -1, 0, "Macromedia Flash$"
html1	db 0, 15, "<!DO CTYPE html"
	db -1, 0, "HTML with DOCTYPE$"
html2	db 0, 5, "<html"
	db -1, 0, "HyperText Markup Language$"
html3	db 0, 5, "<HTML"
	db -1, 0, "HyperText Markup Language$"
mpeg1	db 0, 4, 0,0,1,0xba
	db 12, 4, 0,0,1,0xbb
	db -1, 0, "MPEG1 video$"
mpeg2	db 0, 4, 0,0,1,0xba
	db 14, 4, 0,0,1,0xbb
	db -1, 0, "MPEG2 video$"

bmp	db 0, 2, "BM"
	db 14, 2, 0x28,0
	db -1, 0, "Windows 3 Bitmap image$"
bzip2	db 0, 3, "BZh"
	db -1, 0, "BZIP2 compressed$"
aifc	db 0, 4, "FORM"
	db 8, 4, "AIFC"
	db -1, 0, "AIFF-C audio compressed$"
aiff	db 0, 4, "FORM"
	db 8, 4, "AIFF"
	db -1, 0, "AIFF audio$"
gif87a	db 0, 6, "GIF87a"
	db -1, 0, "GIF 87a image$"
gif89a	db 0, 6, "GIF89a"
	db -1, 0, "GIF 89a image$"
mp3id3	db 0, 3, "ID3"
	db -1, 0, "MP3 with ID3v2 tag$"
tiffi	db 0, 4, "II",0x2a,0
	db -1, 0, "TIFF Intel$"

tiffm	db 0, 4, "MM",0,0x2a
	db -1, 0, "TIFF Mac$"
cab	db 0, 8, "MSCF",0,0,0,0
	db -1, 0, "MS Cabinet archive$"
midi	db 0, 4, "MThd"
	db -1, 0, "MIDI data$"
; exepe	db 0, 2, "MZ"
; 	db 128, 2, "PE"	; offset is stored at [3c] but is often 128
; 	db -1, 0, "Windows binary$"
exemz	db 0, 2, "MZ"
	db -1, 0, "EXE or similar$"	; DOS, Windows, driver, DLL, ...
tex	db 0, 24, "\NeedsTeXFormat{LaTeX2e}"
	db -1, 0, "LaTeX file$"
ogg	db 0, 5, "OggS",0
	db -1, 0, "Ogg audio$"
pdf12	db 0, 8, "%PDF-1.2"
	db -1, 0, "Portable Document Format 1.2$"
pdf13	db 0, 8, "%PDF-1.3"
	db -1, 0, "Portable Document Format 1.3$"
pdf14	db 0, 8, "%PDF-1.4"
	db -1, 0, "Portable Document Format 1.4$"

pdf15	db 0, 8, "%PDF-1.5"
	db -1, 0, "Portable Document Format 1.5$"
pdf16	db 0, 8, "%PDF-1.6"
	db -1, 0, "Portable Document Format 1.6$"
pdf1x	db 0, 6, "%PDF-1"
	db -1, 0, "Portable Document Format 1.x$"
zip	db 0, 4, "PK",3,4
	db -1, 0, "ZIP compressed$"
epscr	db 0, 11, "%!PS-Adobe-"
	db 15, 3, "EPS"
	db -1, 0, "EncapsulatedPostScript$"
pscr2	db 0, 12, "%!PS-Adobe-2"
	db -1, 0, "PostScript 2.x$"
pscr3	db 0, 12, "%!PS-Adobe-3"
	db -1, 0, "PostScript 3.x$"
rar	db 0, 4, "Rar!"
	db -1, 0, "RAR archive$"

avir	db 0, 4, "RIFF"
	db 8, 8, "AVI LIST"
	db 20, 8, "hdlravih"
	db -1, 0, "AVI video$"
midir	db 0, 4, "RIFF"
	db 8, 4, "RMID"
	db -1, 0, "MIDI data RIFF$"
wavpcm	db 0, 4, "RIFF"
	db 8, 7, "WAVEfmt"
	db 20, 2, 1,0
	db -1, 0, "WAV audio PCM$"
realm	db 0, 7, ".RMF",0,0,0
	db -1, 0, "RealMedia video$"
rtf	db 0, 6, "{\rtf1"
	db -1, 0, "RichText$"
sunmu	db 0, 4, ".snd"
	db 12, 4, 0,0,0,1
	db -1, 0, "Sun audio 8bit mu-law$"
sun8	db 0, 4, ".snd"
	db 12, 4, 0,0,0,2
	db -1, 0, "Sun audio 8bit$"
sun16	db 0, 4, ".snd"
	db 12, 4, 0,0,0,3
	db -1, 0, "Sun audio 16bit$"

pcx	db 0, 2, 0x0a,5
	db -1, 0, "PCX v2 image$"
gzip	db 0, 2, 0x1f,0x8b
	db -1, 0, "GZIP compressed$"
wmv	db 0, 4, 0x30,0x26,0xb2,0x75
	db -1, 0, "MS ASF/WMV video$"
elf	db 0, 4, 0x7f,"ELF"
	db -1, 0, "Linux ELF binary$"
x86obj	db 0, 1, 0x80	; pretty weak one!
	db -1, 0, "x86 object file?$"
png	db 0, 8, 0x89,"PNG",13,10,0x1a,10
	db 12, 4, "IHDR"
	db -1, 0, "PNG image$"
jser	db 0, 3, 0xac,0xed,0
	db -1, 0, "Java serialization data$"
jclass	db 0, 4, 0xca,0xfe,0xba,0xbe
	db -1, 0, "Java class$"
msoff	db 0, 8, 0xd0,0xcf,0x11,0xe0,0xa1,0xb1,0x1a,0xe1
	db -1, 0, "Microsoft Office document$"

word2	db 0, 4, 0xdb,0xa5,0x2d,0
	db -1, 0, "Microsoft Word 2$"
utfbom	db 0, 3, 0xef,0xbb,0xbf
	db -1, 0, "UTF-8 with BOM$"
dvi	db 0, 2, 0xf7,2
	db -1, 0, "TeX DVI file$"
jjfif	db 0, 4, 0xff,0xd8,0xff,0xe0
	db 6, 4, "JFIF"
	db -1, 0, "JPEG image JFIF$"
jexif	db 0, 4, 0xff,0xd8,0xff,0xe1
	db 6, 4, "Exif"
	db -1, 0, "JPEG image Exif$"
mp3f2	db 0, 2, 0xff,0xf2
	db -1, 0, "MPEG ADTS layer 3 v2$"
mp3f3	db 0, 2, 0xff,0xf3
	db -1, 0, "MPEG ADTS layer 3 v2$"
mp3fa	db 0, 2, 0xff,0xfa
	db -1, 0, "MPEG ADTS layer 3 v1$"

mp3fb	db 0, 2, 0xff,0xfb
	db -1, 0, "MPEG ADTS layer 3 v1$"
ffff	db 0, 2, 0xff,0xff	; also sort of weak
	db -1, 0, "SYS driver?$"
xml	db 0, 5, "<?xml"
	db -1, 0, "XML file$"
%if larger
xpm	db 0, 9, "/* XPM */"
 	db -1, 0, "X PixMap image$"
%endif
lha	db 2, 3, "-lh"
	db -1, 0, "LHarc lh archive$"
lzh	db 2, 3, "-lz"
	db -1, 0, "LHarch lz archive$"
movfree	db 4, 4, "free"
	db -1, 0, "QuickTime video?$"
mp4v	db 4, 8, "ftypmp42"
	db 16, 8, "mp42avc1"
	db 28, 4, "moov"
	db -1, 0, "QuickTime video ISO MPEG4 mp42avc1$"
mp4a	db 4, 8, "ftypmp42"
	db 16, 8, "mp42mp41"
	db 28, 4, "moov"
	db -1, 0, "QuickTime audio ISO MPEG4 mp42mp41$"

mmdat	db 4, 4, "mdat"
	db 12, 4, "mdat"
	db -1, 0, "QuickTime video mdat$"
mcmov	db 4, 4, "moov"
	db 12, 4, "cmov"
	db -1, 0, "QuickTime video cmov$"
mmvhd	db 4, 4, "moov"
	db 12, 4, "mvhd"
	db -1, 0, "QuickTime video mvhd$"
mwide	db 4, 4, "wide"
	db -1, 0, "QuickTime video wide?$"
kernel	db 0, 8, 0xeb,0x1b,"CONFIG"
	db -1, 0, "Compressed FreeDOS kernel$"
kern0	db 0, 8, 0xeb,0x0d,"CONFIG"	; only tested for #2025
	db -1, 0, "Uncompressed FreeDOS kernel$"
upx1	db 36, 4, "UPX!"	; UPX 2.x compressed COM or CPX
	db -1, 0, "UPX compressed?$"
upx2	db 34, 4, "UPX!"	; UPX 2.x compressed COM or CPX
	db -1, 0, "UPX compressed?$"

; ximod	db 0, 15, "Extended Instrument: "
; 	db -1, 0, "FastTracker XI$"
xmmod	db 0, 17, "Extended module: "	; Soundtracker XM 
	db 37, 1, 0x1a
	db -1, 0, "XM audio$"
st3mod	db 44, 4, "SCRM"
	db -1, 0, "ScreamTracker III$"
%if larger
st2mod1	db 21, 8, "!SCREAM!"
	db -1, 0, "ScreamTracker 2$"
st2mod2	db 21, 8, "BMOD2STM"
	db -1, 0, "ScreamTracker 2$"
%endif
; Protracker magic bytes are at offset 1080, doh! Values:
; M!K! M.K. FLT4 FLT8 or Fasttracker: 4CHN 6CHN 8CHN etc etc
; Actually M.K. seems to be quite popular for really OLD mods.
fli	db 4, 2, 0x11,0xaf
	db -1, 0, "FLI animation$"

align 16
finddta:	; move Disk Transfer Area here for findfirst/findnext
	db 0
